@@ -30,7 +30,7 @@ class Memory:
 
     """
 
-    def __init__(self, threshold=0.0):
+    def __init__(self, threshold=0.5):
         self.threshold = threshold
         self.keys: List[Tensor] = []
         self.values: List[Any] = []
@@ -82,7 +82,7 @@ class Memory:
         value, index = torch.max(sim, 0)
 
         if value.item() < self.threshold:
-            raise IndexError()
+            raise IndexError("No elements in memory")
 
         return index
 
@@ -158,28 +158,10 @@ class Multiset:
 
     @overload
     def __init__(self, dimensions: int, *, device=None, dtype=None):
-        # """Creates an empty multiset of dimensions
-
-        # Args:
-        #     dim (int): number of dimensions of the multiset.
-        # Examples::
-
-        #     >>> M = structures.Multiset(dimensions=10000)
-        # """
         ...
 
     @overload
     def __init__(self, input: Tensor, *, size=0):
-        # """Creates an empty multiset of dimensions
-
-        # Args:
-        #     input (Tensor): tensor representing a multiset.
-        # Examples::
-
-        #     >>> letters = list(string.ascii_lowercase)
-        #     >>> letters_hv = functional.random_hv(len(letters), 10000)
-        #     >>> M = structures.Multiset(input = letters_hv[0])
-        # """
         ...
 
     def __init__(self, dim_or_input: Any, **kwargs):
@@ -259,7 +241,7 @@ class Multiset:
 
     @classmethod
     def from_ngrams(cls, input: Tensor, n=3):
-        """Creates a multiset from the ngrams of a set of hypervectors.
+        r"""Creates a multiset from the ngrams of a set of hypervectors.
 
         See: :func:`~torchhd.functional.ngrams`.
 
@@ -291,7 +273,7 @@ class Multiset:
             >>> M = structures.Multiset.from_tensor(x)
 
         """
-        value = functional.multiset(input, dim=-2)
+        value = functional.multiset(input)
         return cls(value, size=input.size(-2))
 
 
@@ -452,7 +434,7 @@ class HashTable:
 
         """
         value = functional.hash_table(keys, values)
-        return cls(value, size=input.size(-2))
+        return cls(value, size=keys.size(-2))
 
 
 class Sequence:
@@ -467,7 +449,7 @@ class Sequence:
 
     Args:
         input (Tensor): tensor representing a sequence.
-        length (int, optional): the length of the sequence provided as input. Default: ``0``.
+        size (int, optional): the length of the sequence provided as input. Default: ``0``.
 
     Examples::
 
@@ -475,7 +457,7 @@ class Sequence:
 
         >>> letters = list(string.ascii_lowercase)
         >>> letters_hv = functional.random_hv(len(letters), 10000)
-        >>> S = structures.Sequence(letters_hv[0], length=1)
+        >>> S = structures.Sequence(letters_hv[0], size=1)
 
     """
 
@@ -484,11 +466,11 @@ class Sequence:
         ...
 
     @overload
-    def __init__(self, input: Tensor, *, length=0):
+    def __init__(self, input: Tensor, *, size=0):
         ...
 
     def __init__(self, dim_or_input: int, **kwargs):
-        self.length = kwargs.get("length", 0)
+        self.size = kwargs.get("size", 0)
         if torch.is_tensor(dim_or_input):
             self.value = dim_or_input
         else:
@@ -511,7 +493,7 @@ class Sequence:
         """
         rotated_value = functional.permute(self.value, shifts=1)
         self.value = functional.bundle(input, rotated_value)
-        self.length += 1
+        self.size += 1
 
     def appendleft(self, input: Tensor) -> None:
         """Appends the input tensor to the left of the sequence.
@@ -526,7 +508,7 @@ class Sequence:
         """
         rotated_input = functional.permute(input, shifts=len(self))
         self.value = functional.bundle(self.value, rotated_input)
-        self.length += 1
+        self.size += 1
 
     def pop(self, input: Tensor) -> None:
         """Pops the input tensor from the right of the sequence.
@@ -539,7 +521,7 @@ class Sequence:
             >>> S.pop(letters_hv[0])
 
         """
-        self.length -= 1
+        self.size -= 1
         self.value = functional.bundle(self.value, -input)
         self.value = functional.permute(self.value, shifts=-1)
 
@@ -554,7 +536,7 @@ class Sequence:
             >>> S.popleft(letters_hv[1])
 
         """
-        self.length -= 1
+        self.size -= 1
         rotated_input = functional.permute(input, shifts=len(self))
         self.value = functional.bundle(self.value, -rotated_input)
 
@@ -571,10 +553,10 @@ class Sequence:
             >>> S.replace(0, letters_hv[0], letters_hv[1])
 
         """
-        rotated_old = functional.permute(old, shifts=-self.length + index + 1)
+        rotated_old = functional.permute(old, shifts=self.size - index - 1)
         self.value = functional.bundle(self.value, -rotated_old)
 
-        rotated_new = functional.permute(new, shifts=-self.length + index + 1)
+        rotated_new = functional.permute(new, shifts=self.size - index - 1)
         self.value = functional.bundle(self.value, rotated_new)
 
     def concat(self, seq: "Sequence") -> "Sequence":
@@ -591,7 +573,7 @@ class Sequence:
         """
         value = functional.permute(self.value, shifts=len(seq))
         value = functional.bundle(value, seq.value)
-        return Sequence(value, length=len(self) + len(seq))
+        return Sequence(value, size=len(self) + len(seq))
 
     def __getitem__(self, index: int) -> Tensor:
         """Gets the approximate value from given index.
@@ -605,7 +587,7 @@ class Sequence:
             tensor([ 1., -1.,  1.,  ..., -1.,  1., -1.])
 
         """
-        return functional.permute(self.value, shifts=-self.length + index + 1)
+        return functional.permute(self.value, shifts=-self.size + index + 1)
 
     def __len__(self) -> int:
         """Returns the length of the sequence.
@@ -616,7 +598,7 @@ class Sequence:
             0
 
         """
-        return self.length
+        return self.size
 
     def clear(self) -> None:
         """Empties the sequence.
@@ -627,7 +609,7 @@ class Sequence:
 
         """
         self.value.fill_(0.0)
-        self.length = 0
+        self.size = 0
 
     @classmethod
     def from_tensor(cls, input: Tensor):
@@ -659,7 +641,7 @@ class DistinctSequence:
 
     Args:
         input (Tensor): tensor representing a distinct sequence.
-        length (int, optional): the size of the distinct sequence provided as input. Default: ``0``.
+        size (int, optional): the length of the distinct sequence provided as input. Default: ``0``.
 
     Examples::
 
@@ -671,17 +653,19 @@ class DistinctSequence:
         ...
 
     @overload
-    def __init__(self, input: Tensor, *, length=0):
+    def __init__(self, input: Tensor, *, size=0):
         ...
 
     def __init__(self, dim_or_input: int, **kwargs):
-        self.length = kwargs.get("length", 0)
+        self.size = kwargs.get("size", 0)
         if torch.is_tensor(dim_or_input):
             self.value = dim_or_input
         else:
             dtype = kwargs.get("dtype", torch.get_default_dtype())
             device = kwargs.get("device", None)
-            self.value = torch.zeros(dim_or_input, dtype=dtype, device=device)
+            self.value = functional.identity_hv(
+                1, dim_or_input, dtype=dtype, device=device
+            ).squeeze(0)
 
     def append(self, input: Tensor) -> None:
         """Appends the input tensor to the right of the sequence.
@@ -698,7 +682,7 @@ class DistinctSequence:
         """
         rotated_value = functional.permute(self.value, shifts=1)
         self.value = functional.bind(input, rotated_value)
-        self.length += 1
+        self.size += 1
 
     def appendleft(self, input: Tensor) -> None:
         """Appends the input tensor to the left of the sequence.
@@ -713,7 +697,7 @@ class DistinctSequence:
         """
         rotated_input = functional.permute(input, shifts=len(self))
         self.value = functional.bind(self.value, rotated_input)
-        self.length += 1
+        self.size += 1
 
     def pop(self, input: Tensor) -> None:
         """Pops the input tensor from the right of the sequence.
@@ -726,7 +710,7 @@ class DistinctSequence:
             >>> DS.pop(letters_hv[0])
 
         """
-        self.length -= 1
+        self.size -= 1
         self.value = functional.bind(self.value, input)
         self.value = functional.permute(self.value, shifts=-1)
 
@@ -741,7 +725,7 @@ class DistinctSequence:
             >>> DS.popleft(letters_hv[1])
 
         """
-        self.length -= 1
+        self.size -= 1
         rotated_input = functional.permute(input, shifts=len(self))
         self.value = functional.bind(self.value, rotated_input)
 
@@ -759,14 +743,14 @@ class DistinctSequence:
             >>> DS.concat(DS1)
 
         """
-        rotated_old = functional.permute(old, shifts=-self.length + index + 1)
+        rotated_old = functional.permute(old, shifts=self.size - index - 1)
         self.value = functional.bind(self.value, rotated_old)
 
-        rotated_new = functional.permute(new, shifts=-self.length + index + 1)
+        rotated_new = functional.permute(new, shifts=self.size - index - 1)
         self.value = functional.bind(self.value, rotated_new)
 
     def __len__(self) -> int:
-        """Returns the size of the sequence.
+        """Returns the length of the sequence.
 
         Examples::
 
@@ -774,7 +758,7 @@ class DistinctSequence:
             0
 
         """
-        return self.length
+        return self.size
 
     def clear(self) -> None:
         """Empties the sequence.
@@ -784,8 +768,8 @@ class DistinctSequence:
             >>> DS.clear()
 
         """
-        self.value.fill_(0.0)
-        self.length = 0
+        self.value.fill_(1.0)
+        self.size = 0
 
     @classmethod
     def from_tensor(cls, input: Tensor):
@@ -812,13 +796,10 @@ class Graph:
 
     Args:
         dimensions (int): number of dimensions of the graph.
-        directed (bool): decidies if the graph will be directed or not.
+        directed (bool, optional): specify if the graph is directed or not. Default: ``False``.
         dtype (``torch.dtype``, optional): the desired data type of returned tensor. Default: if ``None``, uses a global default (see ``torch.set_default_tensor_type()``).
         device (``torch.device``, optional):  the desired device of returned tensor. Default: if ``None``, uses the current device for the default tensor type (see torch.set_default_tensor_type()). ``device`` will be the CPU for CPU tensor types and the current CUDA device for CUDA tensor types.
-
-    Args:
         input (Tensor): tensor representing a graph hypervector.
-        directed (bool): decidies if the graph will be directed or not.
 
     Examples::
 
@@ -835,7 +816,7 @@ class Graph:
         ...
 
     def __init__(self, dim_or_input: int, **kwargs):
-        self.directed = kwargs.get("directed", False)
+        self.is_directed = kwargs.get("directed", False)
         if torch.is_tensor(dim_or_input):
             self.value = dim_or_input
         else:
@@ -879,16 +860,17 @@ class Graph:
             tensor([-1.,  1., -1.,  ...,  1., -1., -1.])
 
         """
-        if self.directed:
-            return functional.bind(node1, node2)
-        else:
+        if self.is_directed:
             return functional.bind(node1, functional.permute(node2))
+        else:
+            return functional.bind(node1, node2)
 
     def node_neighbors(self, input: Tensor, outgoing=True) -> Tensor:
         """Returns the multiset of node neighbors of the input node.
 
         Args:
             input (Tensor): Hypervector representing the node.
+            outgoing (bool, optional): if ``True``, returns the neighboring nodes that ``input`` has an edge to. If ``False``, returns the neighboring nodes that ``input`` has an edge from. This only has effect for directed graphs. Default: ``True``.
 
         Examples::
 
@@ -896,11 +878,13 @@ class Graph:
             tensor([ 1.,  1.,  1.,  ..., -1., -1.,  1.])
 
         """
-        if self.directed:
+        if self.is_directed:
             if outgoing:
-                return functional.permute(functional.bind(self.value, input), shifts=-1)
+                permuted_neighbors = functional.bind(self.value, input)
+                return functional.permute(permuted_neighbors, shifts=-1)
             else:
-                return functional.bind(self.value, functional.permute(input, shifts=1))
+                permuted_node = functional.permute(input, shifts=1)
+                return functional.bind(self.value, permuted_node)
         else:
             return functional.bind(self.value, input)
 
